package order

import (
	"fmt"
	"fusenapi/constants"
	"fusenapi/model/gmodel"
	"time"
)

type AmountCurrencyReq struct {
	ExchangeRate     int64  `json:"exchange_rate"`     // 换算汇率
	CurrentAmount    int64  `json:"current_amount"`    // 当前金额
	OriginalAmount   int64  `json:"original_amount"`   // 原始金额
	CurrentCurrency  string `json:"current_currency"`  // 当前货币
	OriginalCurrency string `json:"original_currency"` // 原始货币
}

// 汇率换算
func GetAmountCurrency(req *AmountCurrencyReq) gmodel.AmountCurrency {
	if req.CurrentAmount != 0 {
		if req.CurrentCurrency == req.OriginalCurrency {
			req.CurrentAmount = req.OriginalAmount
		} else {
			req.CurrentAmount = req.OriginalAmount * req.ExchangeRate
		}
	}

	return gmodel.AmountCurrency{
		ExchangeRate:     req.ExchangeRate,
		CurrentAmount:    req.CurrentAmount,
		OriginalAmount:   req.OriginalAmount,
		CurrentCurrency:  req.CurrentCurrency,
		OriginalCurrency: req.OriginalCurrency,
	}
}

type GetAmountInfoReq struct {
	ExchangeRate     int64
	Initiate         int64
	Current          int64
	Change           int64
	ChangeRemark     string
	Metadata         map[string]interface{}
	CurrentCurrency  string
	OriginalCurrency string
}

// Change       AmountCurrency         `json:"change,omitempty"`        // 变动金额
// ChangeRemark string                 `json:"change_remark,omitempty"` // 变动备注
// Current      AmountCurrency         `json:"current"`                 // 当前金额
// Initiate     AmountCurrency         `json:"initiate"`                // 初始金额
// Metadata     map[string]interface{} `json:"metadata"`                // 额外明细

func GetAmountInfo(req GetAmountInfoReq) gmodel.AmountInfo {
	return gmodel.AmountInfo{
		Change: GetAmountCurrency(&AmountCurrencyReq{
			ExchangeRate:     req.ExchangeRate,
			CurrentAmount:    req.Change,
			OriginalAmount:   req.Change,
			CurrentCurrency:  req.OriginalCurrency,
			OriginalCurrency: req.OriginalCurrency,
		}),
		ChangeRemark: req.ChangeRemark,
		Current: GetAmountCurrency(&AmountCurrencyReq{
			ExchangeRate:     req.ExchangeRate,
			CurrentAmount:    req.Current,
			OriginalAmount:   req.Current,
			CurrentCurrency:  req.OriginalCurrency,
			OriginalCurrency: req.OriginalCurrency,
		}),
		Initiate: GetAmountCurrency(&AmountCurrencyReq{
			ExchangeRate:     req.ExchangeRate,
			CurrentAmount:    req.Initiate,
			OriginalAmount:   req.Initiate,
			CurrentCurrency:  req.OriginalCurrency,
			OriginalCurrency: req.OriginalCurrency,
		}),
		Metadata: req.Metadata,
	}
}

type GetAmountCurrencyUSDReq struct {
	ExchangeRate     int64  `json:"exchange_rate"`     // 换算汇率
	CurrentAmount    int64  `json:"current_amount"`    // 当前金额
	OriginalAmount   int64  `json:"original_amount"`   // 原始金额
	CurrentCurrency  string `json:"current_currency"`  // 当前货币
	OriginalCurrency string `json:"original_currency"` // 原始货币
}

type GetAmountCurrencyUSDRes struct {
	ExchangeRate     string `json:"exchange_rate"`     // 换算汇率
	CurrentAmount    string `json:"current_amount"`    // 当前金额
	OriginalAmount   string `json:"original_amount"`   // 原始金额
	CurrentCurrency  string `json:"current_currency"`  // 当前货币
	OriginalCurrency string `json:"original_currency"` // 原始货币
}

// 处理金额（美元）
func GetAmountCurrencyUSD(req *GetAmountCurrencyUSDReq) (res GetAmountCurrencyUSDRes) {
	return GetAmountCurrencyUSDRes{
		ExchangeRate: fmt.Sprintf("%.2f", float64(req.ExchangeRate)/1000),
	}
}

func GenerateOrderNumber() string {
	t := time.Now()
	orderNumber := fmt.Sprintf("%d%02d%02d%08d", t.Year(), t.Month(), t.Day(), t.UnixNano()%100000000)
	fmt.Println(orderNumber)
	return orderNumber
}

// 初始化订单状态
func GenerateOrderStatusLink(deliveryMethod int64, noTime time.Time, expectedTime time.Time) []gmodel.OrderStatus {
	var list []gmodel.OrderStatus

	var orderStatus []constants.OrderStatusCode
	if deliveryMethod == constants.DELIVERYMETHODDIRECTMAIL {
		orderStatus = constants.OrderStatusUserDIRECTMAIL
	} else {
		orderStatus = constants.OrderStatusUserCLOUDSTORE
	}
	for _, v := range orderStatus {
		list = append(list, gmodel.OrderStatus{
			StatusCode:  v,
			StatusTitle: constants.OrderStatusMessage[v],
		})
	}
	list[0].Ctime = &noTime
	list[0].Utime = &noTime
	list[len(list)-1].ExpectedTime = &expectedTime
	return list
}

// 获取订单当前状态
func GenerateOrderStatusCurrent() gmodel.OrderStatus {
	return gmodel.OrderStatus{}
}
